/**
  ******************************************************************************
  * @file    LwIP/LwIP_HTTP_Server_Netconn_RTOS/Src/app_ethernet.c
  * @author  MCD Application Team
  * @version V1.0.1
  * @date    29-January-2016
  * @brief   Ethernet specefic module
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2016 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "lwip/opt.h"
#include "main.h"
#include "lwip/dhcp.h"
#include "app_ethernet.h"
#include "app_igmp.h"
#include "ethernetif.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
#ifdef USE_DHCP
#define MAX_DHCP_TRIES  5
__IO uint8_t DHCP_state;
#endif

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Notify the User about the network interface config status
  * @param  netif: the network interface
  * @retval None
  */
void User_notification(struct netif *netif)
{
    if (netif_is_up(netif))
    {
#ifdef USE_DHCP
        /* Update DHCP state machine */
        DHCP_state = DHCP_START;
#endif /* USE_DHCP */
    }
    else
    {
#ifdef USE_DHCP
        /* Update DHCP state machine */
        DHCP_state = DHCP_LINK_DOWN;
#endif  /* USE_DHCP */
    }
}

#ifdef USE_DHCP
/**
* @brief  DHCP Process
* @param  argument: network interface
* @retval None
*/
void DHCP_thread(void const * argument)
{
    struct netif *netif = (struct netif *) argument;
    struct ip_addr ipaddr;
    struct ip_addr netmask;
    struct ip_addr gw;
    uint32_t IPaddress;

    for (;;)
    {
        switch (DHCP_state)
        {
        case DHCP_START:
        {
            netif->ip_addr.addr = 0;
            netif->netmask.addr = 0;
            netif->gw.addr = 0;
            IPaddress = 0;
            dhcp_start(netif);
            DHCP_state = DHCP_WAIT_ADDRESS;
        }
        break;

        case DHCP_WAIT_ADDRESS:
        {
            /* Read the new IP address */
            IPaddress = netif->ip_addr.addr;

            if (IPaddress!=0)
            {
                DHCP_state = DHCP_ADDRESS_ASSIGNED;
                printf("DHCP: got IP %u.%u.%u.%u\n", ip4_addr1(&IPaddress), ip4_addr2(&IPaddress), ip4_addr3(&IPaddress), ip4_addr4(&IPaddress));

                app_igmp_join(IPaddress);

                /* Stop DHCP */
                dhcp_stop(netif);
            }
            else
            {
                /* DHCP timeout */
                if (netif->dhcp->tries > MAX_DHCP_TRIES)
                {
                    DHCP_state = DHCP_TIMEOUT;

                    /* Stop DHCP */
                    dhcp_stop(netif);

                    /* Static address used */
                    IP4_ADDR(&ipaddr, IP_ADDR0 ,IP_ADDR1 , IP_ADDR2 , IP_ADDR3 );
                    IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1, NETMASK_ADDR2, NETMASK_ADDR3);
                    IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
                    netif_set_addr(netif, &ipaddr , &netmask, &gw);
                    printf("DHCP timeout! Using static IP %u.%u.%u.%u\n", IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
                    app_igmp_join(ipaddr.addr);
                }
            }
        }
        break;

        default:
            break;
        }

        /* wait 250 ms */
        osDelay(250);
    }
}
#endif  /* USE_DHCP */

#ifdef USE_DHCP
int GetDhcpState(void)
{
    return DHCP_state;
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
